# Definition of Done for Simantics Platform Releases

1. All Git repositories that are part of the Simantics release train have a branch called `release/x.y.z[.w]` and tag called `x.y.z[.w]`.
2. [A change log entry](https://www.simantics.org/redmine/projects/simantics-platform/wiki/ChangeLog) is compiled from the issues in this release and made available to the general public separately for the platform and for the open products included in the release train.
3. [Roadmap](http://dev.simantics.org/index.php/Roadmap) is up-to-date.
4. [Tutorials](http://dev.simantics.org/index.php/Tutorials) are up-to-date and coherent with the platform.
5. For all new major/minor releases, Wiki documentation is backed up (cloned).

----

# Simantics Platform Release Process

* Create `release/x.y.z[.w]` release stabilisation branch
* Repeat until stable:
  * Develop, test and document components and test products
* Tag the release in repository
* Backup documentation wiki databases
* Build and publish SDK package with and without sources
  * Does not really require any work
* Build Simantics open source products and plug-in components which are a part of the release train
* Update websites to reflect the new release

----

# Released Plug-in Components and Products

There are both plug-in components and products that are part of the _Simantics Release Train_ that shall be released simultaneously to a major or minor Simantics release.

Plug-in components are installable features that are deployed online as P2 repositories for general availability. Products are deployed as ZIP files and made available online in designated locations on simantics.org.

Core components of the release train:
* External third party library dependencies required by the platform - [simantics/third-party.git](https://gerrit.simantics.org/r/gitweb?p=simantics/third-party.git;a=summary)
* The Simantics Platform - [simantics/platform.git](https://gerrit.simantics.org/r/gitweb?p=simantics/platform.git;a=summary)

Plug-in components that are part of the release train:
* Simupedia - [members/simupedia.git](https://gerrit.simantics.org/r/gitweb?p=members/simupedia.git;a=summary)
* FMIL - [simantics/fmil.git](https://gerrit.simantics.org/r/gitweb?p=simantics/fmil.git;a=summary)
* FMI Studio - [members/fmi.git](https://gerrit.simantics.org/r/gitweb?p=members/fmi.git;a=summary)
* Interoperability components - [simantics/interop.git](https://gerrit.simantics.org/r/gitweb?p=simantics/interop.git;a=summary)
* Simantics R binding - [simantics/r.git](https://gerrit.simantics.org/r/gitweb?p=simantics/r.git;a=summary)
* Matlab SCL binding - [simantics/matlab.git](https://gerrit.simantics.org/r/gitweb?p=simantics/matlab.git;a=summary)
* Python SCL binding - [simantics/python.git](https://gerrit.simantics.org/r/gitweb?p=simantics/python.git;a=summary)
* Simantics System Dynamics - [simantics/sysdyn.git](https://gerrit.simantics.org/r/gitweb?p=simantics/sysdyn.git;a=summary)
* Simantics District modelling components - [simantics/district.git](https://gerrit.simantics.org/r/gitweb?p=simantics/district.git;a=summary)
* Simantics 3D modelling components - [simantics/3d.git](https://gerrit.simantics.org/r/gitweb?p=simantics/3d.git;a=summary)
* Simantics Proteus toolset - [gold-members/proteus.git](https://gerrit.simantics.org/r/gitweb?p=gold-members/proteus.git;a=summary)

Products that are part of the release train:
* Simantics Desktop
* Simantics System Dynamics Tool
  * This is Simantics Desktop with Simantics System Dynamics Tool features installed

For simplicity, each of these components are versioned accoring to platform versioning, i.e. for Platform SDK 1.26.0 there will be Simantics Desktop 1.26.0, Sysdyn 1.26.0, and so on.

----

# Release Schedule

* Releases are branched 4 weeks before the set release date.
* During this time no more new features are allowed in the release branch and the existing work shall be stabilized. Work must be focused on bug fixes and documentation.
* Source code changes to release branches are forbidden in the last 2 weeks. Documentation-only changes are still allowed. However, if critical show-stopper problems surface during that time, the changes must be done. After that the release engineering team must decide whether to stick to the original release schedule or whether to delay the release.   

----

# Simantics Platform Release - Step by Step

## Create release branch from selected commit

When release stabilisation starts, branch all relevant repositories like this:

    git clone ssh://<user>@gerrit.simantics.org:29418/simantics/platform.git
    cd platform
    git branch release/x.y.z[.w] <commit>
    git push origin release/x.y.z[.w]

When creating major/minor releases `<commit>` is usually a commit in the `master` branch.
With service releases, branch from an existing `release/*` branch instead.

Instead of doing this clone+branch+push sequence for every SDK repository separately,
please use the included `release-helper.sh` shell script to perform this mechanical work. The script supports many useful basic git commands which all perform the requested operation on all the release train repositories. The operations can be seen from the help text printed by the script when given no arguments:

    Usage: release-helper.sh <version> <branch-name> <user-name> clone|branch|checkout|fetch|list-tags|pull|push|push-tags|remove-tag|status|tag

Always begin by cloning all the repositories for working on 

    ./release-helper.sh x.y.z release/x.y.z your-git-username clone

This will create a directory called `x.y.z` under the current working directory and clone all the repositories under it. The first `<version>` argument is only used for the base working directory `x.y.z`. The second `<branch-name>` argument tells what branch to specify to the action, i.e. `branch`, `checkout`, etc.

Branching and pushing the branches to remote happens as follows:

    ./release-helper.sh x.y.z release/x.y.z your-git-username branch
    ./release-helper.sh x.y.z release/x.y.z your-git-username checkout
    ./release-helper.sh x.y.z release/x.y.z your-git-username push

If you're creating a service release `x.y.w` from `x.y.z` where `w > z`, do this:

    ./release-helper.sh x.y.w release/x.y.z your-git-username clone
    ./release-helper.sh x.y.w release/x.y.z your-git-username checkout
    ./release-helper.sh x.y.w release/x.y.w your-git-username branch
    ./release-helper.sh x.y.w release/x.y.w your-git-username checkout
    # Perhaps make modifications to the branch here and commit to each project separately
    ./release-helper.sh x.y.w release/x.y.w your-git-username push

## Prepare release branch for use

In release 1.31.0 Simantics started using the wonderful tool from Obeo ([https://github.com/mbarbero/fr.obeo.releng.targetplatform](https://github.com/mbarbero/fr.obeo.releng.targetplatform)) to generate `.target` files from `.tpd` files which allow much simpler specification of the target contents and also composition of .tpd files via inclusion.

In the following steps, it is recommended to ensure every `.target` file is up-to-date by regenerating them from `.tpd` files. While doing so, take care to use up-to-date online contents to perform the generation.

### Prepare `.tpd` files in platform repository

0. Ensure you've ran
 
       ./release-helper.sh x.y.z release/x.y.z your-git-username checkout

   to make sure you're working on the release branch before doing anything.

1. Go into the cloned platform repository at `x.y.z/platform`

2. Edit all target platform files in `releng/org.simantics.sdk.build.targetdefinition/`, i.e.
   * `org.simantics.sdk.build.targetdefinition.tpd`
   * `simantics.tpd`

   Replace the following rows in both mentioned files:

   ~~~
   location "http://www.simantics.org/download/master/external-components/maven" {
   location "http://www.simantics.org/download/master/external-components/manual" {
   include "http://www.simantics.org/download/master/org.simantics.sdk.build.targetdefinition.tpd"
   location "http://www.simantics.org/download/master/sdk" {
   ~~~

   with

   ~~~
   location "http://www.simantics.org/download/release/x.y.z[.w]/maven" {
   location "http://www.simantics.org/download/release/x.y.z[.w]/manual" {
   include "http://www.simantics.org/download/release/x.y.z[.w]/org.simantics.sdk.build.targetdefinition.tpd"
   location "http://www.simantics.org/download/release/x.y.z[.w]/sdk" {
   ~~~

3. Edit version number of `org.simantics.sdk` feature in `features/org.simantics.sdk.feature/feature.xml` to `x.y.z`.
   ~~~
   <feature
         id="org.simantics.sdk"
         label="Simantics SDK"
         version="x.y.z"
         provider-name="VTT Technical Research Centre of Finland">
   ~~~

   If the release branch you're setting up is a hotfix-release `x.y.z.w`,
   don't include the `.w` part in this version.

   An example of these changes can be seen in
   [gitweb](https://gerrit.simantics.org/r/gitweb?p=simantics%2Fplatform.git;a=commit;h=5271b0faa98303225d00e6cd838d3fc52488545c)
   or [review 1361](https://gerrit.simantics.org/r/#/c/1361/).

4. Ensure that Gitlab has a release engineering issue for the branched release,
   such as [Simantics 1.34.0 release engineering](https://gitlab.simantics.org/simantics/platform/issues/16)   
   * Make a new release issue with title `$version release engineering`.
   * Add a link to the previous previous release's issue in the new issue description, e.g. `simantics/platform#16`.
   * Add labels `releng` and `$version` to the new issue


5. Commit the changes made

        git commit -a

   with the commit message

        Configured release/x.y.z[.w] branch for SDK builds.

        gitlab #xxxx

   where `#xxxx` is the number of the x.y.z[.w] release engineering issue and push them to remote

        git push origin release/x.y.z[.w]

6. If you are branching from `master`, bump the revision of master right now to start the next release cycle in master.
   An example of these changes can be seen in [gitweb](https://gerrit.simantics.org/r/gitweb?p=simantics%2Fplatform.git;a=commit;h=564ac84a2949b19ce5c1c7c838b575527ec42b09) or in [review 1362](https://gerrit.simantics.org/r/#/c/1362). The changes include:
   * Simantics Desktop splash screen update (version number)
   * Simantics target platform name to state correct development version
   * org.simantics.sdk feature version bump
   * org.simantics.sdk.repository project version bump 

   Commit the changes with the following commit message

       Bumped master target and org.simantics.sdk feature versions to x.y.z[.w].

       gitlab #yyyy

   where `#yyyy` is the number of the next release's release engineering issue.

7. Do the same for simupedia target platform files as was done in task 6.
   for platform target definitions.

### Test Release Train build

Trigger an execution of the [simantics-release-train](https://www.simantics.org/jenkins/job/simantics-release-train/) Jenkins job with parameters
* GERRIT_REFNAME: `release/x.y.z[.w]`
* PUBLISH_ARTIFACTS: `true`

to ensure that all components of the release train build fine and to publish first versions of P2 repositories of everything online.

The release train build is currently never ran automatically. 
However, after following all these steps, whenever changes are pushed/merged to `release/x.y.z[.w]` branch in Gerrit, new Jenkins builds are triggered automatically and they will publish the results (P2 repositories/products) at their designated locations online.

#### [Optional reading] Release train build details

The release train build does two very important steps as the first things,
i.e. build the external dependency P2 repositories for both the
Platform SDK and Simupedia:
~~~
stage('External Dependencies') {
    node {
        build job: 'SDK/Deploy External Components to Web', parameters: params1
        build job: 'Member Components/Simupedia/fi.semantum.simupedia.build.p2.site', parameters: params1
    }    
}
~~~
After that it builds the Platform SDK and Simupedia before building anything else:
~~~
stage('SDK') {
    node {
        build job: 'SDK/Simantics SDK', parameters: params1
        // Note that Simantics SDK builds simupedia-git as well
    }
}
~~~

## Review documentation

Documentation to review:
* [Developer wiki](http://dev.simantics.org/)
* [End-user wiki](https://www.simantics.org/end_user_wiki)
* [Member wiki](https://www.simantics.org/members)

For each wiki page:
* Read through and get authors to fix found problems, such as TODOs or invalid information.

## Review tutorials

* Ensure tutorial wiki documentation at http://dev.simantics.org/index.php/Tutorials is up-to-date with the released platform
* Ensure tutorial projects and product build properly

* com.acme.movie
  - Build with Buckminster, com.acme.movie.product.site.feature

## Tag release/* branches

When the release branches are ready for the release, tag them with the tag `vx.y.z[.w]`:

    ./release-helper.sh x.y.z release/x.y.z your-git-username checkout
    ./release-helper.sh x.y.z release/x.y.z your-git-username pull
    ./release-helper.sh x.y.z release/x.y.z your-git-username tag
    ./release-helper.sh x.y.z release/x.y.z your-git-username push-tags

> Note: release-helper.sh will supply the -m argument to git tag to create an [annotated tag](https://git-scm.com/book/en/v2/Git-Basics-Tagging).
> Only annotated or signed tags can be pushed to Gerrit.

## Backup documentation wiki databases

**NOTE: This step is now deprecated since we are in the process of moving all mediawiki-based documentation into the platform git repository.** 

This step is only necessary for major/minor releases, not for service releases.

The wiki databases to be backed up are:
* [Developer wiki](http://dev.simantics.org/)
* [End-user wiki](https://www.simantics.org/end_user_wiki)

These are MediaWiki installations. The only sane way to "tag" the documentation
is to back up the mysql database backing the wiki. Should the wiki be required
at a later time for some reason, we'll put the documentation up then in a
separate Mediawiki installation.

1. Dump documentation wiki databases using [dump-wikis.sh](./dump-wikis.sh) script.
2. Put the generated backup x.y.z.tar.gz at /var/backup/simantics-releases/x.y.z/wiki/

## Compile change log entry

* Edit the [main page](https://www.simantics.org/redmine/projects/simantics-platform/wiki/ChangeLog) and add a a link for release x.y.z[.w].
* Open the change log page of the previous release, e.g. [1.25.0](https://www.simantics.org/redmine/projects/simantics-platform/wiki/Simantics_1250)
* Open the new link in another browser/tab to start editing the new wiki page
* Edit the previous release's page and copy its wiki source contents over to the new release's page.
* Fix the content to match the new release:
  * Remove the issue content of the previous release
  * Fix release number, release date, release branch link and the link to all issues closed for the release
    * Save new public issue query that lists all issues closed for the release by [starting from the previous release's query](https://www.simantics.org/redmine/projects/simantics-platform/issues?query_id=122)
  * Add filter to query: **Release Notes: Any** to only show issues that have some content in their Release Notes field
  * Export closed issue list as CSV with selected columns only. Open the resulting CSV file in Excel.
  * Use **Data -> Text to Columns** with tab column separation to columnize the result
  * Format the list as a table so that there is only one issue / row
  * Remove all other columns besides: Issue #, Tracker, Release Notes
  * Format the data into a Textile table:
    * Copy to table contents as text into [PSPad](http://www.pspad.com/)
    * Replace (CTRL+H) tabs (`\t`) with `|` with `Regular Expressions` selection checked
    * Use **Insert Text Into Lines..** (ALT-I) to fix line beginnings and ends:
      * Text:
        * At Lines Begin: `|#`
        * At Lines End: `|`
  * Copy the resulting textile table over to the change log page
* Highlight major issues in the list by changing the text background color of the **Type** column:
  * `%{background: lightsalmon}Major Bug%`
  * `%{background: lightgreen}Major Feature%`
  * `%{background: lightgreen}Major Enhancement%`

## Disseminate information about the release

* [Developer Wiki](http://dev.simantics.org): Update roadmap at [http://dev.simantics.org/index.php/Roadmap](http://dev.simantics.org/index.php/Roadmap)
* [Redmine](https://www.simantics.org/redmine/): Post news on the developer/user-visible changes here
* [simantics.org](https://www.simantics.org): Post news on the release and a link to the redmine post
* [Members Wiki](https://www.simantics.org/members/): Update frame plan to reflect the realized dates and link to Redmine news
* [mailto:simantics-developers@simantics.org](mailto:simantics-developers@simantics.org) Send "newsletter" to `simantics-developers@simantics.org:

**Newsletter template:**
~~~
Hello everyone,

Simantics release x.y.z[.w] has been released. Head over to
https://www.simantics.org/redmine/news/<news number>
for the release news.

Best regards,
Simantics Release Engineering Team
~~~

**Redmine news template:**
~~~
Title: Simantics x.y.z[.w] released

Simantics x.y.z[.w] was released on <date>.
Please find change log at: [[simantics-platform:Simantics_xyzw|Simantics x.y.z[.w]]]

Insert some general thoughts on the release...
~~~

----

# TODO

* Incorporate tutorial code in the platform repository as a separate folder to allow platform builds to directly ensure that the tutorial code still builds OK
